@@ -17,7 +17,17 @@ echo $NAME.spec >> MANIFEST
 
 # Build the RPM.
 make
-make test
+perl -MDevel::Cover -e '' 1>/dev/null 2>&1 && USE_COVER=1 || USE_COVER=0
+if [ "$USE_COVER" = "1" ]; then
+  cover -delete
+  HARNESS_PERL_SWITCHES=-MDevel::Cover make test
+  cover
+  mkdir blib/coverage
+  cp -a cover_db/*.html cover_db/*.css blib/coverage
+  mv blib/coverage/coverage.html blib/coverage/index.html
+else
+  make test
+fi
 
 make install
 
